%% DESCRIPTION
% This script uses stocastic simulation to find the generalized leverage
% scores for logistic regression and a given data set. The method is to run 
% several sampled logistic regression models for each observation and
% determining if the prediction of this observation is changed if its label
% is changed.
% CAUTION: Takes a long time to finish even for small data sets.

close all;
clear all;
addpath('..');

run('../plot_settings.m');

% Set data parameters
n = 500;
p = 2;
noiseLevel = 3;

% Generate data and leverage scores and visualize them
[ X, Y, beta ] = generate_data(n,p,noiseLevel,'normal');
Y = (Y > median(Y)) + 1;
figure, scatter(X(:,1),X(:,2),20,Y);

W = zeros(n,n);
for i=1:n
    exponent = [ 1 X(i,:) ] * [ 0 ; beta ];
    W(i,i) = (1/(1 + exp(exponent))) * (1 - 1/(1 + exp(exponent)));
end
H = sqrt(W) * X * ((X'*W*X) \ X' * sqrt(W));
h = diag(H);

% Set algorithm parameters (see thesis for details)
M = 200;        % Subset size
k = 100;         % Simulations per observation


derivative = zeros(n,k);
parfor j=1:n
    if(mod(j,100) == 0)
        disp(j);
    end
    delta = 1;
    for c1=1:k
        % Draw subset of size M
        sample = ceil(n*rand(M,1));
        Xsample = [X(sample,:) ; X(j,:)];
        Ysample = [Y(sample) ; Y(j) ];
        
        % Train model
        beta = mnrfit(Xsample,Ysample);
        value1 = [ 1, Xsample(M+1,:)] * beta;
        
        % Modify the response vector Y
        if(Ysample(M+1) == 1)
            Ysample(M+1) = 2;
            delta = 1;
        else
            Ysample(M+1) = 1;
            delta = -1;
        end
        
        % Re-train the model
        beta = mnrfit(Xsample,Ysample);
        value2 = [ 1, Xsample(M+1,:)] * beta;
        
        % Store the derivative
        derivative(j,c1) = (value2 - value1)/delta;
    end
end

derivativeMean = mean(derivative,2);
derivativeVar = var(derivative,0,2);

% Visualize the results
reportPlot('scatter','Logarithm of stocastic leverage scores for logistic regression',...
            X(:,1),'',...
            X(:,2),'',...
            log(abs(derivativeMean)),''), colorbar;
leverage_visualization(X,derivativeMean);
leverage_visualization(X,abs(derivativeMean));
reportPlot('scatter','Comparison of stocastic and hat matrix leverage',...
        abs(derivativeMean),'Stocastic leverage',...
        h,'Hat matrix leverage');

