%% DESCRIPTION
% Visualize the theoretical bias and variance for \beta from the linear
% regression for a given data set. The theory is first developed by Ma,
% Mahoney and Yu in "A statistical perspective on algorithmic leveraging"
% (2014).
% The bias and variance is calculated for increasing sample sizes,
% illustrating the lowered variance and bias.
% We distinct betwwen the conditional and unconditional results. See thesis
% for more details.

close all;
clear all;

% Set data parameters
n=1000;
p=10;
noiseLevel = 9;
distribution = 'skewed'; % This can be 'normal', 't' or 'skewed'
df = 3;

% Generate data
[ X, y, beta0 ] = generate_data(n,p,noiseLevel,distribution,df);

yNoiseless = X*beta0;
figure(3), hist(yNoiseless.^2,30);
title(mean(yNoiseless.^2));

% Calculate hat leverage scores
H = X*((X'*X) \ X');
h = diag(H);
piLev = h / p;

% Prepare result-varaibles (E = expectation = bias)
sampleSizeVector = 50:50:n; % Define sample sizes to be tested
EConditional = zeros(length(sampleSizeVector),p);
VarConditional = zeros(p,p,length(sampleSizeVector));
EUnconditional = zeros(length(sampleSizeVector),p);
VarUnconditional = zeros(p,p,length(sampleSizeVector));
% Calculate the theoretical bias and variance for several sample sizes
for sIdx=1:length(sampleSizeVector)
    r = sampleSizeVector(sIdx);
    disp(r);
    W0 = diag(r*piLev);
    invXtX = inv(X'*X);
    invXtW0X = inv(X'*W0*X);
    betaWLS = (invXtW0X*X'*W0)*y;
    betaOLS = ((X'*X) \ X')*y;
    
    sigma2 = noiseLevel^2;
    PXW0 = X*invXtW0X*X'*W0;
    
    EConditional(sIdx,:) = betaWLS;
    VarConditional(:,:,sIdx) = (invXtW0X*X'*diag(y-X*betaWLS)*W0)*(diag(y-X*betaWLS)*X*invXtW0X);
    
    EUnconditional(sIdx,:) = beta0;
    VarUnconditional(:,:,sIdx) = sigma2*(invXtW0X*X'*W0^2)*(X*invXtW0X) ...
                        + sigma2*(invXtW0X*X'*diag(ones(n,1)-diag(PXW0))*W0)*(diag(ones(n,1)-diag(PXW0))*X*invXtW0X);

    figure(1);
    ra = sqrt(sum(VarConditional(:,1,sIdx).^2));
    rb = sqrt(sum(VarConditional(:,2,sIdx).^2));
    ang = atan(VarConditional(2,1,sIdx)/VarConditional(1,1,sIdx));
    ellipse(ra,rb,ang,EConditional(sIdx,1),EConditional(sIdx,2));
    
    figure(2);
    ra = sqrt(sum(VarUnconditional(:,1,sIdx).^2));
    rb = sqrt(sum(VarUnconditional(:,2,sIdx).^2));
    ang = atan(VarUnconditional(2,1,sIdx)/VarUnconditional(1,1,sIdx));
    ellipse(ra,rb,ang,EUnconditional(sIdx,1),EUnconditional(sIdx,2));
end

