%% DESCRIPTION
% Show the effect of training SVM with samples of data and using different
% weightings for sampling.
% CONDITIONAL RESULTS

close all;
clear all;

%% Data generation

N = 5000; % Data set size
Ntest = 1000; % Sample size
separation = 2; % Distance between mean of the two classes
axisLimits = [ -5 11 -5 11 ];

SIGMA1 = diag([ 2 ; 0.5 ]);
SIGMA2 = diag([ 3 ; 0.2 ]);

% multivariate normal distribution
% X1 = mvnrnd(zeros(N/2,2),SIGMA1);
% X1test = mvnrnd(zeros(Ntest/2,2),SIGMA1);
% X2 = mvnrnd(separation*ones(N/2,2),SIGMA2);
% X2test = mvnrnd(separation*ones(Ntest/2,2),SIGMA2);
% X = [ X1 ; X2 ];
% Xtest = [ X1test ; X2test ];

% t-distribution
df = 3; % Degree of freedom
X1 = mvtrnd(SIGMA1,df,N/2);
X1test = mvtrnd(SIGMA1,df,Ntest/2);
X2 = mvtrnd(SIGMA2,df,N/2)+kron(ones(N/2,1),[ separation, separation ]);
X2test = mvtrnd(SIGMA2,df,Ntest/2)+kron(ones(Ntest/2,1),[ separation, separation ]);
X = [ X1 ; X2 ];
Xtest = [ X1test ; X2test ];

Y = [ -ones(N/2,1) ; ones(N/2,1) ];
Ytest = [ -ones(Ntest/2,1) ; ones(Ntest/2,1) ];
% Show data
figure();
scatter(X(:,1),X(:,2),10,Y);
axis(axisLimits);

%% Define leverage scores
zeroDistance = sum(X.^2,2);
fiveDistance = sum((X-separation).^2,2);
zeroVector = X - kron(ones(N,1),[ 0, 0 ]);
zeroVector = zeroVector ./ repmat(sqrt(sum(zeroVector.^2,2)),1,2);
fiveVector = X - kron(ones(N,1),[ separation, separation ]);
fiveVector = fiveVector ./ repmat(sqrt(sum(fiveVector.^2,2)),1,2);
H1 = X1*( (X1'*X1) \ X1');
H2 = X2*( (X2'*X2) \ X2');
%W = min(zeroDistance,fiveDistance)./abs(zeroDistance - fiveDistance);
%W = min(zeroDistance,fiveDistance);
%W = [ diag(H1) ; diag(H2) ];
%W = abs(diag(zeroVector * fiveVector'))./(exp(zeroDistance).*exp(fiveDistance));
W = min(zeroDistance,fiveDistance) ./(zeroDistance.*fiveDistance); % BEST RESULT
W = W / sum(W);

figure();
scatter(X(:,1),X(:,2),max(3*N*W,2),W);
axis(axisLimits);

% Set experiment parameters
repetitions = 1000; 
sampleSizeStepSize = 50;
sampleSizeSteps = 20;
errorArray = zeros(repetitions,3);

% Calculate the misclassification for the full SVM model (using all data).
% This is our reference score for the sampled SVM models.
totalSVMStruct = svmtrain(X,Y);
totalPrediction = svmclassify(totalSVMStruct,Xtest);
totalError = sum(abs(totalPrediction-Ytest)) / 2;

% Initialize result variables
meanErrorArray = zeros(sampleSizeSteps,3);
varErrorArray = zeros(sampleSizeSteps,3);

%% Perform experiments
for sam=1:sampleSizeSteps
    sampleSize = sampleSizeStepSize*sam;
    disp(strcat('Sample size: ',num2str(sampleSize)));
    for experiment=1:repetitions
        %% Sampling
        % Uniform sampling
        sample = ceil(N*rand(sampleSize,1));
        uniformSample = X(sample,:);
        uniformY = Y(sample);

        % Non-uniform sampling
        sample = nonuniform_sampling(W,sampleSize);
        leverageSample = X(sample,:);
        leverageY = Y(sample);

        %% Model training

        uniformSVMStruct = svmtrain(uniformSample,uniformY);
        leverageSVMStruct = svmtrain(leverageSample,leverageY);
        uniformPrediction = svmclassify(uniformSVMStruct,Xtest);
        uniformError = sum(abs(uniformPrediction-Ytest)) / 2;
        leveragePrediction = svmclassify(leverageSVMStruct,Xtest);
        leverageError = sum(abs(leveragePrediction-Ytest)) / 2;
        errorArray(experiment,:) = [ totalError, uniformError, leverageError ];

    end

    % Calculate average misclassification and its variance
    meanError = mean(errorArray,1);
    varError = var(errorArray,1);
    meanErrorArray(sam,:) = meanError;
    varErrorArray(sam,:) = varError;
end

%% Visualization
% Show data and SVM separation line for:
% All data
figure();
totalSVMStruct = svmtrain(X,Y,'ShowPlot',1);
axis(axisLimits);
title('Total data');
% Uniform sampled data:
figure();
uniformSVMStruct = svmtrain(uniformSample,uniformY,'ShowPlot',1);
axis(axisLimits);
title('Uniform sample');
% Leverage weighted sampled data:
figure();
leverageSVMStruct = svmtrain(leverageSample,leverageY,'ShowPlot',1);
axis(axisLimits);
title('Leverage sample');

% Show total misclassifications for different sample sizes, comparing
% uniform sampling and leverage weighted sampling
figure();
plot(sampleSizeStepSize:sampleSizeStepSize:sampleSizeSteps*sampleSizeStepSize,meanErrorArray(:,2));
hold on
plot(sampleSizeStepSize:sampleSizeStepSize:sampleSizeSteps*sampleSizeStepSize,meanErrorArray(:,3),'r');
hold off
title('Misclassifications');
legend('Uniform','Leverage');

% Show the variance of total misclassifications for different sample sizes,
% comparing uniform sampling and leverage weighted sampling
figure();
plot(sampleSizeStepSize:sampleSizeStepSize:sampleSizeSteps*sampleSizeStepSize,varErrorArray(:,2));
hold on
plot(sampleSizeStepSize:sampleSizeStepSize:sampleSizeSteps*sampleSizeStepSize,varErrorArray(:,3),'r');
hold off
title('Misclassification variance');
legend('Uniform','Leverage');