%% DESCRIPTION
% This script approximates an image with random samples which are
% interpolated in the reconstruction process.
% The sampling is done iteratively, sampling a fixed number of samples in
% each round, using the difference-image as weighting function for the next
% random draw.
% The approximation is done for a 3d-block of several frames. The idea is
% to use the constant intensity across several frames. The interpolation is
% hence not done for each frame but for a block of frames at once.

close all;
if(~exist('small_mov','var'))
    clear all;
    run('../copy_movie_to_smaller_array.m');
end
addpath('../leverageScores');

%% Preprocessing
% i = 1;
% disp(strcat('Frame',num2str(i)));
% try
%     Igray = rgb2gray(small_mov(i).cdata);
% catch
%     Igray = small_mov(i).cdata(:,:,1);
% end
startFrame = 17;
nFrames = 10;
Igray = zeros(smallHeight,smallWidth,nFrames);
for i=1:nFrames
   Igray(:,:,i) = double(small_mov(startFrame+i-1).cdata(:,:,1)); 
end

%% Settings
initialSampleSize = max(size(Igray,1),size(Igray,2))*nFrames;
maxIterations = min(size(Igray,1),size(Igray,2));
sampleSize = initialSampleSize;
sampleGrowthRate = 2;
pixelCount = size(Igray,2)*size(Igray,1)*nFrames;

[Xgrid, Ygrid, Zgrid] = meshgrid(1:size(Igray,2),1:size(Igray,1),1:nFrames);
weights = ones(pixelCount,1) / pixelCount;
Isample = zeros([size(Igray),nFrames]);
maxDifference = 255;
totalDifference = 255;
cubeOffset = (nFrames-1)*size(Igray,1)*size(Igray,2);
totalSampleIndex = [ 1; size(Igray,1); size(Igray,1)*size(Igray,2)-size(Igray,1)+1; pixelCount; ...
                     cubeOffset+1; cubeOffset+size(Igray,1); cubeOffset+size(Igray,1)*size(Igray,2)-size(Igray,1)+1; cubeOffset+size(Igray,1)*size(Igray,2) ];
totalSampleSize = 8;
statistics = zeros(maxIterations,4); % Column 1: totalSampleSize
                                     % Column 2: totalDifference
                                     % Column 3: maxDifference
                                     % Column 4: elapsedTime


%% Start iterations
iteration = 1;
tic;
while(totalDifference > 3 && iteration < maxIterations)
    %% Sampling
    sampleIndex = nonuniform_sampling(weights,sampleSize);
    totalSampleIndex = [ totalSampleIndex ; sampleIndex ];
    totalSampleSize = totalSampleSize + sampleSize;
    sample = [ Ygrid(totalSampleIndex), Xgrid(totalSampleIndex), Zgrid(totalSampleIndex) ];
    sampleValues = zeros(totalSampleSize,1);
    for j=1:totalSampleSize
        sampleValues(j) = Igray(sample(j,1),sample(j,2),sample(j,3));
        Isample(sample(j,1),sample(j,2),sample(j,3)) = 1;
    end
    
    %% Reconstruction
    Ireconst = griddata(sample(:,2),sample(:,1),sample(:,3),sampleValues,Xgrid,Ygrid,Zgrid);
    Ireconst(isnan(Ireconst)) = 0;
    figure(1), image(Ireconst(:,:,3)*0.3), colormap gray, axis image off, title('Reconstructed image');
    
    differenceImage = abs(Igray - Ireconst);
    maxDifference = max(differenceImage(:));
    totalDifference = sum(differenceImage(:)) / pixelCount;
    figure(2), image(differenceImage(:,:,1)), colormap gray, axis image off, title(strcat('Difference image, totalDifference:',num2str(totalDifference)));
    
    statistics(iteration,:) = [ totalSampleSize*100/pixelCount, totalDifference, maxDifference, toc ];
    
    %% Prepare sampleSize and weights for next iteration
    iteration = iteration + 1;
    sampleSize = initialSampleSize*iteration;
    for j=1:pixelCount
        weights(j) = differenceImage(Ygrid(j),Xgrid(j));
    end
    weights = weights / sum(weights);
    figure(3), image(reshape(weights(1:(size(Igray,1)*size(Igray,2))),size(Igray,1),size(Igray,2))*pixelCount*2), axis image off, title('Weights');
end

figure(4), image(Isample(:,:,1)*60), colormap gray, axis image off, title(strcat('Samples ',num2str(totalSampleSize),'=',num2str(totalSampleSize*100/pixelCount),'%'));
reconstFig = figure(5);
frame = 1;
while(~strcmp(get(reconstFig,'CurrentCharacter'),'q'))
    image(Ireconst(:,:,mod(frame,nFrames)+1)*0.3), colormap gray, axis image off, title('Press "q" to exit');
    frame = frame + 1;
    waitforbuttonpress;
end