function [ mask ] = roi_select( mov, frame )
%ROI_SELECT Select a Region Of Interest
%   Input:  - mov, movie structure
%           - frame, for which the ROI is returned (must be > 0 and <
%           nFrames)
%   Output: - mask, an image of logicals, indicating the ROI

    if(frame < 1 || frame >= length(mov))
        error('Frame must be > 0 and < total frame count');
    end

    frame1 = mov(frame).cdata;
    frame2 = mov(frame+1).cdata;

    % Load SIFT-toolbox
    run('../vlfeat-0.9.18/toolbox/vl_setup.m');
    % Preprocessing
    if(size(frame1,3) == 3)
        frame1 = rgb2gray(frame1);
    end
    if(size(frame2,3) == 3)
        frame2 = rgb2gray(frame2);
    end

    % Calculate SIFT-features
    [fa,da] = vl_sift(single(frame1));
    [fb,db] = vl_sift(single(frame2));
    [matches, ~ ] = vl_ubcmatch(da,db,1.8);

    % Initialize return-variables
    mask = false(size(frame1));

    % Get displacement of matched SIFT-features
    xDisplacement = fa(1,matches(1,:)) - fb(1,matches(2,:));
    yDisplacement = fa(2,matches(1,:)) - fb(2,matches(2,:));
    displacement = [ xDisplacement' yDisplacement' ];
    % Discard features with a large displacement (probably mismatches)
    displacement(displacement > 10) = NaN;
    % Return if there is not enough data
    if(sum(~isnan(sum(displacement,2))) < 3)
        return;
    end

    % K-Means clustering:
    clusterIDX = kmeans(displacement,2,'emptyaction','singleton');
    cluster1IDX = find(clusterIDX == 1);
    cluster2IDX = find(clusterIDX == 2);

    % Use SVM for smoothing of clusters and building the mask
    if(length(clusterIDX) == size(displacement,1))
        SIFTpositions = [ fa(1,matches(1,:))', fa(2,matches(1,:))' ];
        SVMModel = svmtrain(SIFTpositions,clusterIDX,'kernel_function','rbf','rbf_sigma',0.5);
        clusterIDX = svmclassify(SVMModel,SIFTpositions);
        cluster1IDX = find(clusterIDX == 1);
        cluster2IDX = find(clusterIDX == 2);

        [ Xgrid, Ygrid ] = meshgrid(1:size(frame1,2),1:size(frame1,1));
        mask(:) = svmclassify(SVMModel,[ Xgrid(:), Ygrid(:)]) > 1;
    end
    % Define the smaller cluster as the region of interest, since we assume
    % that the background is larger than the object of interest
    if(length(cluster1IDX) < length(cluster2IDX))
        mask = ~mask;
    end
end

